/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
*
*  MODULE :             TableOfBits
*  SCOPE :              Allocation through table of bits
*  FILE :               tableofbits.hxx
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

/*=
[DESCRIPTION :]

   This class is a tool similar to fd_set. It is a table of bits. Each bit
   can be set or clear. Interface of the class allows to set, clear, or test
   sequence of bits. A method allows to find the first sequence of clear bits.
   This method can be used to do allocation.
=*/

#ifdef __GNUG__
#pragma interface
#endif
#ifndef __tableofbits_HXX
#define __tableofbits_HXX

/* IMPORTS *******************************************************************/

#ifndef __stdlib_H
#define __stdlib_H
#include <stdlib.h>
#endif

#ifndef __stdio_H
#define __stdio_H
#include <stdio.h>
#endif

/*****************************************************************************/

#define NUMBITS(typ) (sizeof(typ)*8)
#define yoo_alignSup(val,base) ((val-1)/base*base+base)


class TableOfBits {

  public :

  // Ctor of a table of bits. If no srctab is given, one is allocated
  TableOfBits(int size, char* srctab = NULL) : _size(size) {
    _tab = (srctab == NULL) ?
      new char[yoo_alignSup(size/NUMBITS(char), 4)] : srctab ;
    _doalloc = (srctab == NULL) ? 1 : 0 ;
  }
  TableOfBits(const TableOfBits& t) { *this = t ; } 

  ~TableOfBits() { if (_doalloc) delete _tab ; }

  int  set(int pos, int len) ;   // Set a sequence of bits
  int  clear(int pos, int len) ;   // clear a sequence of bits
  void zero() ; // fill the tab with clear bits
  void one() ; // fill the tab with set bits 
  int  isset(int pos, int len) const ; // Return if the sequence is set
  int  isclear(int pos, int len) const ; // Return if the sequence is clear
  int  look(int pos, int len) const ; // Search a sequence of cleared bits
  TableOfBits& operator=(const TableOfBits& t) ; // Copy a table of bits
  
  private :

  void set(int i) {_tab[i/NUMBITS(char)] |= (1 << (i%NUMBITS(char)));}
  void clear(int i) {_tab[i/NUMBITS(char)] &= ~(1 << (i%NUMBITS(char)));}
  int  isset(int i) const {
    return (_tab[i/NUMBITS(char)] & (1<<(i%NUMBITS(char))));
  }
  int  isclear(int i) const {return !this->isset(i) ; }
  int  nextclear(int pos) const ; 

  int  _doalloc ;
  int  _size ;  
  char* _tab ;
} ;

#endif
